#ifndef LOGGER_H
#define LOGGER_H

#include <string>

// Naplózó osztály (absztrakt).
class Logger
{
public:
    // Üzenet szintje.
    enum Level
    {
        DEBUG,
        INFO,
        WARNING,
        ERROR,
        CRITICAL
    };

    // Virtuális destruktor.
    // A virtualitás fontos, hogy polimorfizmus esetén a dinamikus típus destruktora meghívódjon.
    virtual ~Logger() {}

    // Naplózás.
    void log(const std::string& message, Level level);

protected:
    // Az egyetlen konstruktor nem publikus, hanem védett, így csak a leszármazott tudja meghívni.
    // Mivel az osztályban van tisztán virtuális metódus, ezért az osztály amúgy sem lenne példányosítható.
    Logger(Level minLevel);

    // Tényleges naplózás. (Absztrakt metódus, felül kell definiálni.)
    virtual void doLog(const std::string& message, Level level) = 0;

    // Statikus függvény az enumerációs értékek szöveggé alakítására.
    static std::string levelToStr(Level level);

private:
    // Minimum üzenet szint, ami felett naplóz a példány.
    Level minLevel;
};

#endif // LOGGER_H
