#include <iostream>
#include "Logger.h"
#include "ConsoleLogger.h"
#include "FileLogger.h"

using namespace std;

int main()
{
    // Konzolos naplózó objektum létrehozása.
    ConsoleLogger cl(Logger::INFO);
    cl.log("Log message 1", Logger::INFO);
    cl.log("Log message 2", Logger::DEBUG);
    cl.log("Log message 3", Logger::WARNING);

    // A polimorfizmus működik referenciával.
    Logger& r = cl;
    r.log("Log message 4", Logger::INFO);
    r.log("Log message 5", Logger::DEBUG);
    r.log("Log message 6", Logger::WARNING);

    // A polimorfizmus működik pointerrel is.
    Logger* p = new ConsoleLogger(Logger::INFO);
    p->log("Log message 7", Logger::INFO);
    p->log("Log message 8", Logger::DEBUG);
    p->log("Log message 9", Logger::WARNING);
    delete p;

    // A pointer statikus típusa változatlan, de dinamikus típusa megváltozhat.
    p = new FileLogger("app.log", Logger::DEBUG);
    p->log("Log message 10", Logger::INFO);
    p->log("Log message 11", Logger::DEBUG);
    p->log("Log message 12", Logger::WARNING);

    // A leszármazottak speciális metódusinak használatához cast-olnunk kell a típust.
    FileLogger *fl = dynamic_cast<FileLogger*>(p);
    // A cast-olás sikeres volt, ha az eredmény nem null pointer. (Referencia esetén bad_cast exception, mivel null reference nem létezik.)
    if(fl)
    {
        cout << "Log file name: "
            << dynamic_cast<FileLogger*>(p)->getPath()
            << endl;
    }

    delete p;
    return 0;
}
