#include <cmath>
#include <stdexcept>
#include "PrimGenerator.h"

PrimGenerator::PrimGenerator()
{
    primes.push_back(2);
}

unsigned long PrimGenerator::generate(unsigned int n)
{
    if(n == 0)
        throw std::out_of_range("n must be positive.");
    
    if(primes.size() < n)
        createPrimes(n);

    return primes[n - 1];
}

void PrimGenerator::createPrimes(unsigned int n)
{
    primes.reserve(n);
    
    while(primes.size() < n)
    {
        unsigned long test = primes.back() + 1;
        while(!isPrime(test))
        {
            if(++test == 0)
                throw std::out_of_range("Numeric overflow.");
        }
        primes.push_back(test);
    }
}

bool PrimGenerator::isPrime(unsigned long number)
{
    for(unsigned int i = 0;
        i < primes.size() && primes[i] <= sqrt(number);
        ++i)
    {
        if(number % primes[i] == 0)
            return false;
    }
    
    return true;
}
